#ifndef _HATPTASKLIST_H
#define _HATPTASKLIST_H

#include<map>

#include "hatpNode.hh"


/**
 * Representation of sequence of nodes called TaskList.
 * A TaskList corresponds to a "decomposition" of a tree node.
 */
class hatpTaskList {

	private:
		std::map<unsigned int, hatpNode*> nodes;
		std::vector<std::pair<int, unsigned int> > ranks;
		int maxRank;
	
	protected:

		/**
		 * Used by the constructor to compute the rank of each node in the TaskList.
		 * To each node is associated a rank (0 for root) indicating its position in the tasklist
		 * @see hatpTaskList()
		 * @param cnode The node for which to compute rank
		 */
		void computeNodeRank(hatpNode* cnode);
	
	public:

		/**
		 * Constructor of a TaskList.
		 * @param n The list of nodes composing the TaskList
		 */
		hatpTaskList(std::map<unsigned int, hatpNode*> n);

		/**
		 * Constructor of a TaskList.
		 * @param n A list of nodes (nodes composing the TaskList must be included)
		 * @param v A list of nodes ID composing the TaskList
		 */
		hatpTaskList(std::map<unsigned int, hatpNode*> n, std::vector<unsigned int> v);

		/**
		 * Empty destructor.
		 */
		~hatpTaskList();

		/**
		 * Get the size of the current TaskList.
		 * @return The size of the current TaskList
		 */
		size_t size() const;

		/**
		 * Get The list of nodes of the current TaskList.
		 * @return A list of nodes composing the TaskList
		 */
		std::map<unsigned int, hatpNode*> getNodes() const;

		/**
		 * Get the maximum node ranking.
		 * @see computeNodeRank()
		 * @return The max rank for the current TaskList
		 */
		unsigned int getMaxRank() const;

		/**
		 * Get all the nodes of a specified rank for the current TaskList.
		 * @param r the rank
		 * @return A list of all nodes of rank r
		 */
		std::vector<hatpNode*> getNodesOfRank(int r) const;

		/**
		 * Get The rank of a given node ID.
		 * @param n The node ID
		 * @return The rank of the specified ID node or -1
		 */
		int getRankOfNode(unsigned int n) const;

		/**
		 * Test if a TaskList is a decomposition of the current one.
		 * @param TL The TaskList to test
		 * @return true if TL is a decomposition of the current TaskList, false otherwise
		 */
		bool isTaskListDecomposition(hatpTaskList* TL) const;

		/**
		 * Get the node for which the TaskList given in parameter is a decomposition from.
		 * @param TL The TaskList which is the decomposition of the current one
		 * @return The node which is root of TL
		 */
		hatpNode* getRootTaskListDecomposition(hatpTaskList* TL) const;

		/**
		 * Get the TaskList under a text form.
		 * @return A string representing the TaskList
		 */
		std::string toString() const;

};

#endif
